#package Listing ;

#####################################################################
#               Module de suivi des communication                   #
# AUTEUR : Guillaume Lehmann                                        #
# Pour tout renseignement, lehmann@free.fr                          #
# Ce logiciel est sous licence GPL v3 (http://www.gnu.org)          #
# v 0.1 du 18/10/2009                                               #
# v 0.2 du 20/03/2010                                               #   
# v 0.3 du 27/03/2010                                               #   
# v 0.4 du 09/05/2010                                               #   
# v 0.5 du 16/05/2010                                               #   
# v 0.6 du 18/05/2010                                               #   
#####################################################################



print '[Fonctions "Listing" inclues !]'."\n" ; 


###############################
#           FUNCTIONS
# Listing_findComm()
# Listing_addComm()
# Listing_addTime()
# Listing_delComm()
###############################




############################################################################
## Create a new listing to manage active communications
## Args: none
## Returns: $this
## Comment:
##    Constructor
############################################################################
#sub new {  
#    print "Entre dans la mthode new\n" ;
#    my $this = {};
#    bless $this ;
#    my ($poubelle, $arg1, $arg2) = @_ ;
#        
#    $this->{TabIndex}[$arg1] = $arg2 ;
#    $this->{TabCommCycleNow} = 0 ;
#    $this->{TabCommCycleInit} = 0 ;
#    $this->{TabCommMess} = "" ;
#    
#    return $this ;
#}

###########################################################################
# Find the index to a specific communication
# Args: ($callIDFct)
#   $numCommFct : communication number
# Returns: $indexCommFct : index communication number, -1 if not found.
# Comment:
#    None
###########################################################################
sub Listing_findComm {
    my $callIDFct = $_[0] ;
    my $indexCommFct = -1 ;

    # Browse the table to find the communication number.
    for (my $compteurFct = scalar(@TabComm); $compteurFct >= 0; $compteurFct --) {
	if ($TabComm[$compteurFct] eq $callIDFct) {
	    $indexCommFct = $compteurFct ;
	}
    }

    return($indexCommFct) ;
}


###########################################################################
# Add a new communication in the listing
# Args: ($toAddressFct, $toPortFct, $callIDFct)
#   $toAddressFct : Called address.
#   $toPortFct : Called port.
#   $callIDFct : INVITE  Call-ID: field.
# Returns: $indexComm : the index affected to this communication.
# Comment:
#    None
###########################################################################
sub Listing_addComm {
    my $toAddressFct = $_[0] ;
    my $toPortFct = $_[1] ;
    my $callIDFct = $_[2] ;
    

    # We attribute for "communication index" the first empty table's cell (so the last cell + 1).
    my $indexCommFct = scalar(@TabComm) ;
    
    # Complete every moniroting tables.
    $TabComm[$indexCommFct] = $callIDFct ;
    $TabCommCycleInit[$indexCommFct] = 2 ; # Initial value.
    $TabCommCycleNow[$indexCommFct] = 2 ; # The timer will do 2 cycles before action.
    
    # If the port To: isn't indicate, it will be the default value (5060).
    if (!$toPortFct) {
	$toPortFct = 5060 ;
    }
    # Record called's informations.
    $TabCommDest[$indexCommFct] = "/udp/$toAddressFct/$toPortFct/" ;

    #print "[DEBUG]Listing.pm, Ajout de comm. n$indexCommFct\n" ;

    return($indexCommFct) ;
}


###########################################################################
# Add times to one communication's timer
# Args: ($numCommFct, $timeFct, $messageFct, $destinataireFct)
#   $numCommFct : communication number
#   $timeFct : number of cycle to add
#   $messageFct : SIP message to store
#   $destinataireFct : receiver address 
# Returns: 1
# Comment:
#    None
###########################################################################
sub Listing_addTime {
    my $numCommFct = $_[0] ;
    my $timeFct = $_[1] ;
    my $messageFct = $_[2] ;
    my $destinataireFct = $_[3] ;

    $TabCommCycleNow[$numCommFct] = $timeFct ;
    $TabCommCycleInit[$numCommFct] = $timeFct ;
    if ($messageFct) {$TabCommMess[$numCommFct] = $messageFct ;}
    if ($destinataireFct) {$TabCommDest[$numCommFct] = $destinataireFct ;}

    return 1 ;
}


###########################################################################
# Delete a communication from the listing.
# Args: ($numCommFct)
#   $numCommFct : communication number
# Returns: 1
# Comment:
#    None
###########################################################################
sub Listing_delComm { # anciennement sub SupprListingComm {
    my $indexCommFct = $_[0] ;
    # print "[DEBUG]Listing.pm, Suppression de comm. n$indexCommFct\n" ;
    my $lastElementFct = scalar(@TabComm) - 1 ;
    # Pack tables on the element to delete.
    for (my $compteurFct = $indexCommFct; $compteurFct < $lastElementFct; $compteurFct ++) {
	$TabComm[$compteurFct]          = $TabComm[$compteurFct + 1] ;
	$TabCommCycleNow[$compteurFct]  = $TabCommCycleNow[$compteurFct + 1] ;
	$TabCommMess[$compteurFct]      = $TabCommMess[$compteurFct + 1] ;
	$TabCommDest[$compteurFct]      = $TabCommDest[$compteurFct + 1] ;
	$TabCommCycleInit[$compteurFct] = $TabCommCycleInit[$compteurFct + 1] ;
	$TabAuth[$compteurFct]          = $TabAuth[$compteurFct + 1] ;
	$TabCommBranchIngress[$compteurFct] = $TabCommBranchIngress[$compteurFct + 1] ;
	$TabCommBranchEgress[$compteurFct] = $TabCommBranchEgress[$compteurFct + 1] ;
    }

    # Delete the last table element.
    delete $TabComm[$lastElementFct] ;
    delete $TabCommCycleNow[$lastElementFct] ;
    delete $TabCommMess[$lastElementFct] ;
    delete $TabCommDest[$lastElementFct] ;
    delete $TabCommCycleInit[$lastElementFct] ;
    delete $TabAuth[$lastElementFct] ;
    delete $TabCommBranchIngress[$lastElementFct] ;
    delete $TabCommBranchEgress[$lastElementFct] ;

    return 1 ;
}
1 ;
