#!/usr/bin/perl


#############################################################
#                  FONCTIONS UTILES                         #
# AUTEUR : Guillaume Lehmann                                #
# Pour tout renseignement, lehmann@free.fr                  #
# Ce logiciel est sous licence GPL v3 (http://www.gnu.org)  #
# v 0.1 du 07/06/2008     v0.21 du 19/03/2010               #
# v 0.2 du 06/07/2008     v0.22 du 20/03/2010               #
# v 0.3 du 07/07/2008     v0.23 du 24/03/2010               #
# v 0.4 du 21/07/2008     v0.24 du 27/03/2010               #
# v 0.5 du 09/08/2008     v0.25 du 07/05/2010               #
# v 0.6 du 16/09/2008     v0.26 du xx/03/2010               #
# v 0.7 du 23/09/2008     v0.27 du xx/03/2010               #
# v 0.8 du 27/09/2008     v0.28 du xx/03/2010               #
# v 0.9 du 30/09/2008     v0.29 du xx/03/2010               #
# v 0.10 du 03/11/2008    v0.30 du xx/03/2010               #
# v 0.11 du 09/04/2009    v0.31 du xx/03/2010               #
# v 0.12 du 12/04/2009    v0.32 du xx/03/2010               #
# v 0.13 du 14/04/2009    v0.33 du xx/03/2010               #
# v 0.14 du 25/04/2009    v0.34 du xx/03/2010               #
# v 0.15 du 23/06/2009    v0.35 du xx/03/2010               #
# v 0.16 du 28/06/2009    v0.36 du xx/03/2010               #
# v 0.17 du 20/09/2009    v0.37 du xx/03/2010               #
# v 0.18 du 18/10/2009    v0.38 du xx/03/2010               #
# v 0.19 du 05/03/2010    v0.39 du xx/03/2010               #
# v 0.20 du 18/03/2010    v0.40 du xx/03/2010               #
#############################################################


require "Packet.pm" ;
require "Listing.pm" ;


#############################
#         FUNCTIONS
# LCR_Discri()
# TempoControl()
# AuthCalc()
# AuthGeneration()
# Taxation()
# ScriptInterface() (Not yet entirely include in pabx.pl)
#############################




###########################################################################
# LCR and discrimination case study
# Args: ($numSourceFct, $numDestFct)
#   $numSourceFct : caller phone number
#   $numDestFct : called phone number
# Returns: Next SIP proxy or the called IP address or NOTFOUND or the interdiction decision
# Comment:
#    None
###########################################################################
sub LCR_Discri {
    # Rcupration des numros source et destination.
    my $numSourceFct = $_[0] ;
    my $numDestFct = $_[1] ;
    my $conditionEntFct = '' ;
    my $conditionFct = '' ;
    my $decisionFct = '' ;
    my $authFct = '' ;
    my $optionFct = '' ;
    my $optAuthFct = '' ;
    my $optNextFct = '' ;
    my $optTimeFct = '' ;
    my $optTimeOutFct = '' ;
    my @tabLocalTimeFct = () ;
    my @tabOptTimeFct = () ;
    

    # On recherche la condition la plus proche de la situation actuelle et  la premre que l'on voit on s'arrte de chercher et on regarde la dcision.
    open (LCRDISCRI, "LCR_Discri.conf") ;
    while ($conditionEntFct = <LCRDISCRI>) {

	# Passer  la ligne suivante si la ligne commence par un # (commentaire).
	if ($conditionEntFct =~ /\#/) {
	    next ;
	}
	
	chomp($conditionEntFct) ;
	
	($conditionFct, $decisionFct, $optionFct) = $conditionEntFct =~ /(.*) Decision:(LOCAL|AUTORISE|INTERDIT)\s?(.*)?/ ;
	
	# Vrifier que l'on a la bonne correspondance pour les numros appellant->appell.
	if ("$numSourceFct --> $numDestFct" =~ /^$conditionFct$/) {
	    # Si on a trouv la correspondance, on vrifie si nous avons l'option Time: de dfinie, on s'assure alors que l'on est dans le bon crneau.
	    if (($optTimeFct) = $optionFct =~ /Time:(\S*)/) {
		# Rcupration de la date courante et de la dfinition du crneau autoris.
		@tabLocalTimeFct = localtime() ;
		(@tabOptTimeFct) = $optTimeFct =~ /(\d{1,2}):(\d{1,2})\-(\d{1,2}):(\d{1,2});(\d)\-(\d);(\d{1,2})\-(\d{1,2});(\d{1,2})\-(\d{1,2})/ ;
		
		# Mois/Jour/Heure_Minute/JourSemaine
		# On rajoute +1 pour le mois courant. Pour comparer 2 heures entre elles, on convertit en minutes : $Heure*60 + $Minute.
		if ($tabOptTimeFct[8] <= $tabLocalTimeFct[4]+1 and $tabLocalTimeFct[4]+1 <= $tabOptTimeFct[9]
		    and $tabOptTimeFct[6] <= $tabLocalTimeFct[3] and $tabLocalTimeFct[3] <= $tabOptTimeFct[7]
		    and ($tabOptTimeFct[0]*60 + $tabOptTimeFct[1]) <= ($tabLocalTimeFct[2]*60 + $tabLocalTimeFct[1])
		    and ($tabLocalTimeFct[2]*60 + $tabLocalTimeFct[1]) <= ($tabOptTimeFct[2]*60 + $tabOptTimeFct[3])
		    and $tabOptTimeFct[4] <= $tabLocalTimeFct[6] and $tabLocalTimeFct[6] <= $tabOptTimeFct[5]) {
		    
		    print "[DEBUG] pabxFct.pl On est dans le crneau horaire. On sort de la boucle.\n" ;
		    last ;		   
		}
		else {
		    print "[DEBUG] pabxFct.pl Nous sommes hors du crneau horaire.\n" ;
		    next ;
		}
	    } # Fin "if (($optTimeFct) = $optionFct =~ /Time:(\S*)/)".
	    
	    # Option Time: non dfinie. On sort de la boucle car on n'a quand mme la correspondance pour l'appellant et l'appell.	
	    last ;
	}
    }
    
    # Fermeture du fichier.
    close (LCRDISCRI) ;
    
    # Rcupration des options :
    ($optAuthFct) = $optionFct =~ /Auth:(\S*)/ ;
    ($optNextFct) = $optionFct =~ /Next:(\S*)/ ;
    
    # print "[DEBUG] pabxFct.pl, LCR_Discri : $conditionFct / $decisionFct / $optNextFct / $optAuthFct\n" ; 

    # Si l'appel est local, on renvoie l'adresse indique dans le tableau $TabAdresse{}. Si aucune adresse n'a t enregistre (cas d'un poste local non enregistr), on renvoie "NOTFOUND".
    if ($decisionFct eq 'LOCAL') {
	if ($TabAdresse{$numDestFct}) {
	    return $TabAdresse{$numDestFct} ;
	}
	else {
	    return 'NOTFOUND' ;
	}
    }
    # Si la dcision est d'autoriser l'appel, on retourne l'adresse IP du saut suivant. Sinon on renvoie la dcision (INTERDIT).
    elsif ($decisionFct eq 'AUTORISE') {
	return $optNextFct ;
    }
    else {
	return $decisionFct ;
    }
}


###########################################################################
# Time managing and watch outdated communication transmission for retransmission or delete it.
# Args: ()
# Returns: -
# Comment:
#    Work in a independant thread. Use shared variables.
###########################################################################
sub TempoControl {
    while(1) {

        # Un cycle vaut 500 ms.
	usleep(500000) ;
	# Supprimer tous les suivis qui n'ont pas d'infos.
	for (my $compteurFct = 0; $compteurFct < scalar(@TabComm); $compteurFct ++) {
	    if ($TabCommMess[$compteurFct] eq '' or $TabCommDest[$compteurFct] eq '') {
		# print "[DEBUG] pabxFct.pl, Suppression de $TabComm[$compteurFct]\n" ; 
		Listing_delComm($compteurFct) ;
	    }
	}


	# Parcourir le listing des communications pour enlever un cycle de temps  chacun. Pour chaque expiration de cycle, renvoyer le paquet et recharger le timer sauf si on arrive au bout et donc que l'on enlve les infos de suivi ce qui aura pour effet d'tre supprim  la prochaine tempo.
	for (my $compteurFct = 0; $compteurFct < scalar(@TabComm); $compteurFct ++) {
#	    # print "[DEBUG] pabxFct.pl, dcrmentation : $TabComm[$compteurFct] : $TabCommCycleNow[$compteurFct]\n" ;
	    # Dcrmenter d'un cycle (0.5 seconde).
	    $TabCommCycleNow[$compteurFct] = $TabCommCycleNow[$compteurFct] - 1 ;
	    
	    # Si on est  <= 0 pour cette communication et qu'il y a eu trop de rmission (dernire fois, temps d'attente >= 32), alors clore le tout et effacer le suivi.
	    if (($TabCommCycleNow[$compteurFct] <= 0) && ($TabCommCycleInit[$compteurFct] >= 32)) {
		# Si c'est un INVITE, alors 480 No Response  l'appellant et CANCEL  l'appell. Ensuite, vider le suivi.
# 		if ($TabCommMess[$compteurFct] =~ /INVITE sip:/) {
# 		    # Rcuprer les Via:, From:, To:, Call-ID: et CSeq:.
# 		    (my $DiversCompletFct, my $RefHAuthorizationFct, my $RefHCall_IDFct, my $RefHContactFct, my $Content_TypeFct, my $Content_LengthFct, my $RefHCSeqFct, my $RefHFromFct, my $RefHMax_ForwardsFct, my $Subscription_StateFct, my $RefHToFct, my $RefTabViaFct, my $RefTabRRouteFct, my $RefTabRouteFct, my $LigneEtatFct, my $CorpsMessageFct) = CutHeader($TabCommMess[$compteurFct]) ;
# 		    # Drfrencement des variables rcupres de la fonction.
# 		    my %HAuthorizationFct = %$RefHAuthorizationFct ;
# 		    my %HCall_IDFct = %$RefHCall_IDFct ;
# 		    my %HContactFct = %$RefHContactFct ;
# 		    my %HCSeqFct = %$RefHCSeqFct ; 
# 		    my %HFromFct = %$RefHFromFct ;
# 		    my %HToFct = %$RefHToFct ;
# 		    my @TabViaFct = @$RefTabViaFct ;
# 		    my %HMax_ForwardsFct = %$RefHMax_ForwardsFct ;
# 		    my @TabRRouteFct = @$RefTabRRouteFct ; 
# 		    my @TabRouteFct = @$RefTabRouteFct ;
		
# 		    # Recherche de l'adresse  qui envoyer le 480 (c'est celle indique dans le 2ime Via: en partant du haut de la pile).
# 		    ($poubelle, my $AdresseFct, my $PortFct) = $TabViaFct[1] =~ /Via: SIP\/2\.0\/(UDP|TCP|SCTP) (\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}):?(\d*)?.*/ ;
# 		    # Enlever le Via: du haut de la pile et envoyer  l'adresse indique dans le second Via: (adresse rcupre ci-dessus).
# 		    shift(@TabViaFct) ;
# 		    my $ParaViaFct = join("\r\n", @TabViaFct) ;
# 		    # Composition du paragraphe Record-Route:.
# 		    my $ParaRRouteFct = join("\r\n", @TabRRouteFct) ;
# 		    # Composition du message
# 		    my $Message480Fct = "SIP/2.0 480 No Response\r
# $ParaViaFct\r
# $HFromFct{'From'}\r
# $HToFct{'To'}\r
# $ParaRRouteFct\r
# $HCall_IDFct{'Call_ID'}\r
# $HCSeqFct{'CSeq'}\r
# $DiversCompletFct\r
# Content-Length: 0
# \r\n" ;
		
# 		    # Si aucun port n'est indiqu, on utilise le port par dfaut qui est le 5060.
# 		    if (!$PortFct) {
# 			$PortFct = 5060 ;
# 		    }
# 		    # Envoi de la rponse
# 		    my $clientFct = IO::Socket::INET->new( Proto    => 'udp',
# 							   PeerPort => $PortFct,
# 							   PeerAddr => $AdresseFct) ;
# 		    die "[ERROR] Impossible de charger le client thread_480 : $!" unless $clientFct ;
# 		    # Envoi du message
# 		    $clientFct->send($Message480Fct) or die "[ERROR] Erreur lors de l'envoi du Message480Fct : $!" ;
# 		    # Fermeture du client.
# 		    close($clientFct) ;
# 		    $TabSession[$compteurFct] = "480" ;
# 		} # Fin du "if ($TabCommMess[$compteurFct] =~ /INVITE sip:/)"
		
		# Vider le suivi.
		$TabCommDest[$compteurFct] = '' ;
		
	    } # Fin du "if (($TabCommCycleNow[$compteurFct] <= 0) && ($TabCommCycleInit[$compteurFct] >= 32))"
	    # Si on est  <= 0 pour cette communication mais que l'on peut faire encore des tentatives, alors rmission.
	    elsif (($TabCommCycleNow[$compteurFct] <= 0) && ($TabCommCycleInit[$compteurFct] <= 32)) {

		# Remettre le message et recharger le timer au double.
		print "[WARN ] [T]Rmission engage\n" ;
#		print "[DEBUG] Paquet rmis : $TabCommMess[$compteurFct]\n" ;
		# Rcupration du destinataire.
		(my $destProtoFct, my $destAdresseFct, my $destPortFct) = $TabCommDest[$compteurFct] =~ /\/(.*)\/(.*)\/(.*)\// ;
		my $clientFct = IO::Socket::INET->new( Proto    => $destProtoFct,
						       PeerPort => $destPortFct,
						       PeerAddr => $destAdresseFct) ;
		warn "[ERROR] [T]Impossible de charger le client dans le thread : $!" unless $clientFct ;
		
		## Rmission du message.
		$clientFct->send($TabCommMess[$compteurFct]) if ($clientFct && $destProtoFct && $destPortFct && $destAdresseFct) or warn "Erreur lors de la rmission : $!" ;
		
		## Fermeture du client.
		close($clientFct) ;
		
		
		## Recharger le timer au double de la valeur initiale
		# print "[DEBUG] pabxFct.pl, Rarmement timer au double\n" ;
		$TabCommCycleInit[$compteurFct] = $TabCommCycleInit[$compteurFct] * 2 ;
		$TabCommCycleNow[$compteurFct] = $TabCommCycleInit[$compteurFct] ;
	    } # Fin du "elsif (($TabCommCycleNow[$compteurFct] <= 0) && ($TabCommCycleInit[$compteurFct] <= 32))"
	    
	} # Fin du "for (my $compteurFct = 0; $compteurFct < scalar(@TabComm); $compteurFct ++)"
	
    } # Fin du while(1).
}


###########################################################################
# Calculate MD5 hash response for the SIP authentication
# Args: ($methodFct, $passFct, $usernameFct, $realmFct, $ncFct, $nonceFct, $cnonceFct, $uriFct, $qopFct)
#   $methodFct : SIP method.
#   $passFct : password.
#   $usernameFct : username
#   $realmFct : realm
#   $ncFct : nc
#   $nonceFct : nonce
#   $cnonceFct : cnonce
#   $uriFct : uri
#   $qopFct : qop
# Returns: $hrepFct : MD5 hash which correpond to the previous arguments.
# Comment:
#    None
###########################################################################
sub AuthCalc {
    my $methodFct = $_[0] ;
    my $passFct = $_[1] ;
    my $usernameFct = $_[2] ;
    my $realmFct = $_[3] ;
    my $ncFct = $_[4] ;
    my $nonceFct = $_[5] ;
    my $cnonceFct = $_[6] ;
    my $uriFct = $_[7] ;
    my $qopFct = $_[8] ;


    # Les requtes ACK utilisent les mthodes de la requte initiale.
    if ($methodFct eq 'ACK') {
	$methodFct = 'INVITE' ;
    }
    
    # Voir RFC2617 pour le dtail de l'algorithme.
    ## Composition de a1 et a2.
    my $a1Fct = "$usernameFct:$realmFct:$passFct" ; 
    my $a2Fct = "$methodFct:$uriFct" ;

    # print "[DEBUG] pabxFct.pl, AuthCalc(...) => $a1Fct\n$a2Fct\n" ;

    ## Calcul des hashs.
    my $ha1Fct = md5_hex($a1Fct) ;
    my $ha2Fct = md5_hex($a2Fct) ;
    
    ## Composition de la rponse.
    my $repFct = "$ha1Fct:$nonceFct:$ncFct:$cnonceFct:$qopFct:$ha2Fct" ;
    # Certaines piles SIP utilisent ceci (quand le qop n'est pas dfini) : $repFct = "$ha1Fct:$nonceFct:$ha2Fct" ;

    ## Calcul du hash de la rponse.
    my $hrepFct = md5_hex($repFct) ;

    # Retourner le hash de la rponse.
#    print "[DEBUG] pabxFct.pl, Rponse : $hrepFct\n" ;
    return ($hrepFct) ;
}


###########################################################################
# WWW-Authenticate field composition.
# Args: ()
# Returns: The WWW-Authenticate string, the nonce and the opaque.
# Comment:
#    None
###########################################################################
sub AuthGeneration {
    # Cration du nonce.
    my $var1Fct = Packet_compoTagBranchNum('t') ;
    my $var2Fct = Packet_compoTagBranchNum('t') ;
    my $nonceFct = $var1Fct.$var2Fct ;
    # Cration de l'opaque.
    $var1Fct = Packet_compoTagBranchNum('t') ;
    $var2Fct = Packet_compoTagBranchNum('t') ;
    my $opaqueFct = $var1Fct.$var2Fct ;

    return ("Digest realm=\"$DOMAINE\", qop=\"auth\", nonce=\"$nonceFct\", opaque=\"$opaqueFct\", stale=\"false\", algorithm=MD5", $nonceFct, $opaqueFct) ;
}


###########################################################################
# Tax managment
# Args: ($actionFct, $fromFct, $toFct)
#   $actionFct : Indicate if we mark a communication establishement ("new") or the end ("end").
#   $fromFct : Caller identifier.
#   $toFct : Called identifier.
# Returns: -
# Comment:
#    None
###########################################################################
# Gestion de la taxation.
sub Taxation {
    my $actionFct = $_[0] ;
    my $fromFct = $_[1] ;
    my $toFct = $_[2] ;
    (my $dateSecondeFct, my $dateMinuteFct, my $dateHeureFct, my $dateJourFct, my $dateMoisFct, my $dateAnneeFct,,,) = localtime() ;

    $dateAnneeFct += 1900 ;
    $dateMoisFct += 1 ; # +1 sinon a un mois de retard
    
    # Inscrire dans le fichier de taxation la tentative de communication.
    if ($actionFct eq 'try') {
	open (FICHIERTAX, '>> Taxation.txt') ;
	print FICHIERTAX "[TAX] $dateJourFct/$dateMoisFct/$dateAnneeFct $dateHeureFct:$dateMinuteFct.$dateSecondeFct Tentative $fromFct --> $toFct\n" ;
	close (FICHIERTAX) ;
    }	
    # Inscrire dans le fichier de taxation le dbut d'une communication.
    if ($actionFct eq 'new') {
	open (FICHIERTAX, '>> Taxation.txt') ;
	print FICHIERTAX "[TAX] $dateJourFct/$dateMoisFct/$dateAnneeFct $dateHeureFct:$dateMinuteFct.$dateSecondeFct Comm_dbut $fromFct --> $toFct\n" ;
	close (FICHIERTAX) ;
    }
    # Inscrire dans le fichier de taxation la fin d'une communication.
    elsif ($actionFct eq 'end') {
	open (FICHIERTAX, '>> Taxation.txt') ;
	print FICHIERTAX "[TAX] $dateJourFct/$dateMoisFct/$dateAnneeFct $dateHeureFct:$dateMinuteFct.$dateSecondeFct Comm_fin $fromFct --> $toFct\n" ;
	close (FICHIERTAX) ;
    }
}


###########################################################################
# Execute user's script
# Args: ($actionFct, $callerFct, $calledFct)
#   $actionFct : Indicate in which state we are in the communication process.
#   $calleRFct : Indicate who initiate the communication.
#   $calleDFct : Indicate which number is called.
# Returns: -
# Comment:
#    It is the same things that AGI in Asterisk.
###########################################################################
# Execute a script.
sub ScriptInterface {
    my $actionFct = $_[0] ;
    my $calleRFct = $_[1] ;
    my $calleDFct = $_[2] ;  
    
    my $scriptNameFct = $actionFct."_".$calleRFct."_".$calleDFct ;
    if (-s $scriptNameFct) {
	system ($scriptNameFct) ;
    }
}
1;







# # S'authentifier sur les passerelles SIP. 
# sub AuthExterne {

#     # Crer tous les lments ncessaires  la composition du message REGISTER.
#     my $brancheFct = Packet_compoTagBranchNum("b") ;
#     my $tagFct = Packet_compoTagBranchNum() ;
#     my $idFct = ComposTagBranchNum() ;

#     # Composer le message REGISTER.
#     $messageREGISTER = "REGISTER sip:$ADRESSEGW:$PORTGW SIP/2.0
# Via: SIP/2.0/UDP $ADRESSESRV:$PORT;branch=$brancheFct\r
# Max-Forwards: 70\r
# From: <sip:$NUMGW\@$ADRESSEGW>\r
# To: <sip:$NUMGW\@$ADRESSEGW>;tag=$tagFct\r
# Call-ID: $idFct\r
# Contact: *\r
# Expires: 0\r
# Content-Length: 0\r
# \n\r" ;


#     # Lancer les REGISTER vers toutes les passerelles SIP.


# }
# RFC 3263 : Comment trouver le serveur SIP  qui faire suivre la requte SIP.


# Numros de rponses existants : 415, 420, 421.




# Depuis le nom, rcuprer l'adresse IP.
# my $toto = inet_ntoa(inet_aton('free.fr'));
# my $toto = 'aa$toto' ;
# print ">>$toto<<" ;
# if ($toto =~ /a\$/) {
#    print "\nmerde\n" ;
# }$TabCommCycleNow[100]
# die() ;


## Joujou avec de le perl objet
#my $paq = Packet->Packet_cutHeader2($message) ;
#my $v = $paq->getLigneEtat() ;
#print ">>$v<<\n" ;
#my $u = $paq::ligneEtatFct2 ;
#print ">>Ne passe pas : $u<<\n" ;
#$TabCommCycleNow[100] = "testmain" ;
#testons() ;
#print ">>$TabCommCycleNow[100]<<\n" ;
#die() ;
